import { useState, useEffect } from 'react';

import s from './Timer.module.css';

interface IRenderProps {
  seconds: number;
  refresh: number;
}

interface TimerProps {
  time: number;
  flag: string;
  setFlag: React.Dispatch<React.SetStateAction<string>>;
  refresh: number;
  render: ({ seconds, refresh }: IRenderProps) => JSX.Element;
}

const Timer = ({
  time,
  flag,
  setFlag,
  refresh,
  render: TimerPresentation,
}: TimerProps) => {
  const [timeLeft, setTimeLeft] = useState<number>(0);

  useEffect(() => {
    setTimeLeft(time);
  }, [time]);

  useEffect(() => {
    const handleInterval = () =>
      timeLeft > 0 && flag === 'Pause' && setTimeLeft(timeLeft - 1);

    const idInterval = setInterval(handleInterval, refresh);
    return () => clearInterval(idInterval);
  }, [timeLeft, flag, refresh]);

  const handleTimer = () => {
    switch (flag) {
      case 'Start':
        setFlag('Pause');
        break;
      case 'Pause':
        setFlag('Start');
        break;
      default:
        break;
    }
  };

  return (
    <div className={s.timerWrapper}>
      <button className={s.timerBtn} onClick={handleTimer}>
        {flag}
      </button>
      <p>refresh speed {refresh} ms</p>
      <p>{timeLeft} sec</p>
      <TimerPresentation seconds={timeLeft} refresh={refresh} />
    </div>
  );
};

export default Timer;
